"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.LoggerLevel = void 0;
const ansi_colors_1 = __importDefault(require("ansi-colors"));
var LoggerLevel;
(function (LoggerLevel) {
    LoggerLevel["ALL"] = "trace";
    LoggerLevel["TRACE"] = "trace";
    LoggerLevel["DEBUG"] = "debug";
    LoggerLevel["INFO"] = "info";
    LoggerLevel["WARN"] = "warn";
    LoggerLevel["ERROR"] = "error";
    LoggerLevel["FATAL"] = "fatal";
    LoggerLevel["SILENT"] = "silent";
})(LoggerLevel = exports.LoggerLevel || (exports.LoggerLevel = {}));
const mapping = {
    all: 0,
    trace: 2,
    debug: 4,
    info: 6,
    warn: 8,
    error: 10,
    fatal: 11,
    silent: 12,
};
const mappingValue = Object.keys(mapping).reduce((sum, e) => {
    sum[mapping[e]] = e;
    return sum;
}, {});
class Logger {
    context;
    indent;
    level;
    indentString = '';
    indentStringLog = '';
    get levelName() {
        return mappingValue[this.level];
    }
    constructor(level, context = '', indent = 0) {
        this.context = context;
        this.indent = indent;
        this.setLevel(level);
        if (this.indent)
            this.updateIndent(this.indent);
    }
    is(level) {
        const lv = typeof level === 'number' ? level : mapping[level];
        return lv !== mapping.silent ? (this.level <= lv) : (this.level === lv);
    }
    clone(context) {
        return new Logger(this.level, context || this.context, this.indent);
    }
    updateIndent(indent) {
        this.indent = indent;
        this.indentString = this.getIndentString(this.indent);
        this.indentStringLog = this.getIndentString(this.indent, '╎ ');
    }
    addIndent(indent = 1) {
        this.updateIndent(this.indent + indent);
    }
    removeIndent(indent = 1) {
        this.updateIndent(this.indent - indent);
    }
    getIndentString(indent, fill = '╎╴') {
        let str = new Array(indent).fill(`${fill[0]} `).join('');
        if (str.length) {
            str = str.substring(0, str.length - 1) + fill[1];
        }
        return ansi_colors_1.default.gray.dim(str);
    }
    formatWithoutIndent(msg, level) {
        const sp = msg ? ' ' : '';
        switch (level) {
            case LoggerLevel.INFO:
                return `${ansi_colors_1.default.green(`✔`)}${sp}${msg}`;
            case LoggerLevel.WARN:
                return `${ansi_colors_1.default.yellow(`⚠️`)}${sp}${msg}`;
            case LoggerLevel.ERROR:
                return `${ansi_colors_1.default.red(`✘`)}${sp}${msg}`;
            case LoggerLevel.FATAL:
                return `${ansi_colors_1.default.red(`❌`)}${sp}${msg}`;
            case LoggerLevel.TRACE:
                return `${ansi_colors_1.default.magenta.italic(`⑃${sp}${msg}`)}`;
        }
        return `${msg}`;
    }
    format(msg, level) {
        return `${this.indentString}${this.formatWithoutIndent(msg, level)}`;
    }
    setLevel(level) {
        const lv = typeof level === 'number' ? level : mapping[level];
        if (lv !== undefined && lv !== null)
            this.level = lv;
    }
    print(...args) {
        console.log(...args);
    }
    debugBlock(isStart, msg = this.context || '', ...prms) {
        if (!this.is(LoggerLevel.DEBUG))
            return;
        if (!isStart)
            msg = '';
        console.log(`${this.indentString}%s ${ansi_colors_1.default.gray.dim.italic.underline(msg)}`, ansi_colors_1.default.gray.dim(isStart ? '┌' : '└'), ...prms);
        if (isStart)
            this.addIndent();
        else
            this.removeIndent();
    }
    log(msg, ...prms) {
        if (!this.is(LoggerLevel.SILENT)) {
            msg.split('\n').forEach(msg => {
                console.log(`${this.indentStringLog}${msg}`, ...prms);
            });
        }
    }
    info(msg, ...prms) {
        if (this.is(LoggerLevel.INFO)) {
            msg.split('\n').forEach(msg => {
                console.log(`${this.format(msg, LoggerLevel.INFO)}`, ...prms);
            });
        }
    }
    debug(msg, ...prms) {
        if (this.is(LoggerLevel.DEBUG)) {
            msg.split('\n').forEach(msg => {
                console.log(`${this.indentString}${ansi_colors_1.default.gray(msg)}`, ...prms);
            });
        }
    }
    warn(msg, ...prms) {
        if (this.is(LoggerLevel.WARN)) {
            msg.split('\n').forEach(msg => {
                console.log(`${this.format(msg, LoggerLevel.WARN)}`, ...prms);
            });
        }
    }
    trace(msg, ...prms) {
        if (this.is(LoggerLevel.TRACE)) {
            msg.split('\n').forEach(msg => {
                console.log(`${this.format(msg, LoggerLevel.TRACE)}`, ...prms);
            });
        }
    }
    error(msg, ...prms) {
        if (this.is(LoggerLevel.ERROR)) {
            msg.split('\n').forEach(msg => {
                console.log(`${this.format(msg, LoggerLevel.ERROR)}`, ...prms);
            });
        }
    }
    fatal(msg, ...prms) {
        if (this.is(LoggerLevel.ERROR)) {
            msg.split('\n').forEach(msg => {
                console.log(`${this.format(msg, LoggerLevel.FATAL)}`, ...prms);
            });
        }
    }
}
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map